
#ifndef _global_h
#define _global_h

typedef struct {
   int id;			/* id of this node                          */
   double x, y, z, t;		/* the real coordinates of the Node         */
   int dx, dy;                  /* the devise coordinates of the Node       */
   int subdomain;		/* in which subdomain this Node belongs     */
   int interf;		        /* if the node is an interface or not       */
   int boundary;		/* if this node belongs to the boundary     */
   int nadjNodes;		/* number of adjacent Nodes to this one     */
   int *adjNodes;		/* the adjacent Nodes to this one           */
   int nadjElements;		/* number of adjacent Elements to this Node */
   int *adjElements;		/* the adjacent Elements to this Node       */
} Node;


typedef struct {
   int *vertices;               /* the vertices of one fase   */
   int boundary;                /* if this face is a boundary */
   double *ExtentsList;         /* xEXTmin, yEXTmin, zEXTmin, */
   /* ^--- used for shading */  /* xEXTmax, yEXTmax, zEXTmax  */
} Face;


typedef struct {
   int id;			/* id of this Element			      */
/*
   int nadjNodes;		   the number of adjacent nodes of an element
                                   ^-- this entry equal to "MaxNodesElement"  */
   int *adjNodes;		/* the vertices of one element                */
   int nadjElements;            /* number of adjacent Element to this one     */
   int *adjElements;            /* the adjacent Element to this one           */
} Element;


typedef enum _Dec_Algorithm {
   CM_clust,
   Diagonal,
   Horizontal,
   Vertical,
   PtimesQ,
   Ker_lin,
   GGP_alg,
   GGP_recur,
   Hierarchical,
   Continuous
} Dec_Algorithm;		/* available decomposition algorithms */


typedef enum _Guess {
   GuessNone,
   GuessCurrent,
   GuessRandom
} Guess;			/* initial decomposition guesses */


typedef struct _View {		/* x specifics */
   Display *display;
   Window window;
   Pixmap pixmap;
   int screen;
   GC gc;
   Dimension xwidth, xheight;
   double scale_x, scale_y, trans_x, trans_y;
   XFontStruct *font_struct;
   Colormap colormap;
   Pixel foreground, background;
} View, *ViewPtr;


typedef struct _PaletteInfo {
   Widget paltoplevel;          /* toplevel palette window                 */
   Widget palwin;               /* palette graphics workspace window       */
   Pixel *pixels;               /* array of pixels for color allocation    */
   int PalCellWidth, PalCellHeight, PalRows, PalCols; /* various constants */
   ViewPtr v;                   /* x specifics                             */
   int ncolors;                 /* # of colors in palette                  */
   Pixel start;                 /* starting pixel of allocated color cells */
   Boolean first;               /* used for one-time initializations       */
} PaletteInfo, *PaletteInfoPtr;


typedef struct _Domain {
   int   id;                    /* numeric id of this dubdomain		   */
   int   ndomnodes;             /* number of nodes in subdomain		   */
   int  *domnodes;              /* nodes of this domain			   */
   Node *NodeList;              /* the node info for this domain           */
   int   nelements;             /* number of elements in the subdomain     */
   int  *elements;              /* elements assigned to domain		   */
   Element *ElementList;        /* the element info for this domain        */
                                /*                                         */
   int *EdgeList;               /* the EdgeList info for this domain       */
   int *BoundaryEdgeList;       /* the BoundaryEdgeList info this domain   */
   Face *FaceList;              /* the FaceList info this domain           */

   Dimension xwidth, xheight;	/* width and height of the pixmaps         */
   double scale_x, scale_y,
          trans_x, trans_y;
} Domain, *DomainPtr;


typedef struct _DecInfo {
   Widget worktoplevel;		/* toplevel drawing decomposition window   */
   Widget nodeWidget;		/* used to display # of nodes              */
   Widget elementWidget;	/* used to display # of elements           */
   Widget interWidget;		/* used to display # of interfaces         */
   Widget domainsWidget;	/* used to display number of domains       */
   Widget SelectionWidget;	/* used to display current selection       */
   Widget timeWidget;		/* used to display decompositin time       */
   Widget workwin;		/* decomposition graphics workspace window */
   Widget dectoplevel;		/* toplevel decomposition window           */
   Widget guessbox;		/* used to disable guess selection         */
   Widget algtoggle[10];	/* algorithm selection toggles             */
   Widget guesstoggle[2];	/* guess selection toggles                 */
                                /*                                         */
   Widget *wdomains;            /* the widgets used for splitting object   */
   Widget *dom_w_canvas;        /* the canvas used for splitting object    */
   Domain *domains;             /* array holding domain information        */
   Window *dwindows;            /* windows for the splitting object        */
   Pixmap *dpixmaps;            /* pixmaps for the splitting object        */

   Dec_Algorithm algorithm;	/* algorithm to be used for decomposition  */
   Guess guess;			/* guess to be fed as input if applicable  */
                                /*                                         */
   int getdecomposition;        /* flag                                    */
   int nsubdoms;                /* number of subdomains                    */
   int interfaces;              /* number of interfaces                    */
   int CurrentDomain;		/* selected domain                         */
   int ppq, qpq;		/* used for PtimesQ algorithm              */

   PaletteInfoPtr p;
   ViewPtr v;

} DecInfo, *DecInfoPtr;		/* decomposition info */

#endif /* _global_h */
