
#include "unix.h"
#include "x.h"
#include "xt.h"
#include "xpex.h"
#include "xmu.h"
#include "color.h"
#include "externs.h"
#include "globals.h"

extern XtAppContext app_context;
extern Display     *display;
extern Colormap     cmap;
extern Colormap     colormap;
extern XColor       colors[64];
extern Widget       hsvColor;
extern Widget       hlsColor;
extern Widget       cieColor;

RGBf rgbf_values;
RGB  rgb_values;
HSV  hsv_values;
HLS  hls_values;
CIE  cie_values;

int xpos=0, ypos=0;
int buttons_down = 0;
int bars_locked = 0;
double locked_top = 1.0, red_top, green_top, blue_top;
float pass_value;
int last_pos;
Boolean do_change = TRUE;
Pixel original_background;

XtActionProc hsv_Expose(  );
XtActionProc hls_Expose(  );
XtActionProc cie_Expose(  );

PEXRenderer           mixed_rd;
PEXRendererAttributes mixed_rdVals;

extern Widget toplevel, mixingForm, mixedColor;

extern Widget redLocked, greenLocked, blueLocked, title;
extern Widget redLabel, greenLabel, blueLabel;
extern Widget redIntensity, greenIntensity, blueIntensity, tripleValue;
extern Widget redScroll, greenScroll, blueScroll, lockedScroll;
extern Widget hueScroll, satScroll, valScroll, lgtScroll;
extern Widget hueLabel, satLabel, valLabel, lgtLabel;
extern Widget XScroll, YScroll, ZScroll;
extern Widget XLabel, YLabel, ZLabel;
extern Widget quit;

struct app_resources {
    Boolean silent;
    String text;
    String format;
} prog_res;

#define RADDEG( _a )  ( ( float )( _a ) * 180.0 / 3.1415926 )
#define DEGRAD( _a )  ( ( float )( _a ) * 3.1415926 / 180.0 )

void
lock_toggle( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    Arg args[1];
    int button = WhichButton( params[0] );

    args[0].name = XtNbackground;
    if ( button & bars_locked ) {
        args[0].value = original_background;
        bars_locked -= button;
        if ( !bars_locked )
            XtSetSensitive( lockedScroll, FALSE );
    } else {
        switch ( button ) {
        case RED:
            args[0].value = colors[PURE_RED].pixel;
            break;
        case GREEN:
            args[0].value = colors[PURE_GREEN].pixel;
            break;
        case BLUE:
            args[0].value = colors[PURE_BLUE].pixel;
            break;
        default:
            return;
            /* NOT REACHED */
        }
        bars_locked += button;
        XtSetSensitive( lockedScroll, TRUE );
    }
    move_lock(  );
    XtSetValues( w, ( ArgList )args, 1 );
}


void ChangeBorder( button, now_up )
int button;
Boolean now_up;
{
    Widget scrollbar;

    switch ( button ) {
    case RED:
        scrollbar = redScroll;
        button = now_up ? PURE_RED : PURE_BLACK;
        break;
    case GREEN:
        scrollbar = greenScroll;
        button = now_up ? PURE_GREEN : PURE_BLACK;
        break;
    case BLUE:
        scrollbar = blueScroll;
        button = now_up ? PURE_BLUE : PURE_BLACK;
        break;
    default:
        return;
        /* NOTREACHED */
    }
    XtVaSetValues( scrollbar, XtNborderColor, colors[button].pixel, NULL );
}

void
ChangeLabel( button, value )
int button;
double value;
{
    Widget label;
    String buffer;

    buffer = ( String ) malloc( 10 );
    switch ( button ) {
    case RED:
        label = redLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case GREEN:
        label = greenLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case BLUE:
        label = blueLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case HUE:
        label = hueLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case SAT:
        label = satLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case VAL:
        label = valLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case LGT:
        label = lgtLabel;
        sprintf( buffer, "%.3lf", 1.-value );
        break;
    case XVAL:
        label = XLabel;
        sprintf( buffer, "%.3lf", value );
        break;
    case YVAL:
        label = YLabel;
        sprintf( buffer, "%.3lf", value );
        break;
    case ZVAL:
        label = ZLabel;
        sprintf( buffer, "%.3lf", value );
        break;
    default:
        return;
        /* NOTREACHED */
    }
    XtVaSetValues( label, XtNlabel, buffer, NULL );
}

void
border( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    Boolean reset = ( Boolean )( *num_params == 1 && strcmp( params[0], "reset" ) == 0 );

    if ( w == redScroll )
        ChangeBorder( RED, reset );
    else {
        if ( w == greenScroll )
            ChangeBorder( GREEN, reset );
        else {
            if ( w == blueScroll )
                ChangeBorder( BLUE, reset );
            else {
                if ( bars_locked & RED ) ChangeBorder( RED, reset );
                if ( bars_locked & GREEN ) ChangeBorder( GREEN, reset );
                if ( bars_locked & BLUE ) ChangeBorder( BLUE, reset );
            }
        }
    }
}


int
WhichButton( name )
String name;
{
    if ( strcmp( name, "red" ) == 0 )
        return RED;
    if ( strcmp( name, "blue" ) == 0 )
        return BLUE;
    if ( strcmp( name, "green" ) == 0 )
        return GREEN;
    return 0;
}


void
update_triple( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    extern Boolean workingRGB, workingHSV, workingHLS, workingCIE;
    extern int depth;

    Arg args[1];
    char hexvalue[10];
    float value, saturation, hue;

    sprintf( hexvalue, prog_res.format,
        COLOR( MIX_COLOR,red ), COLOR( MIX_COLOR,green ),
        COLOR( MIX_COLOR,blue ) );
    XtVaSetValues( tripleValue, XtNlabel, hexvalue, NULL );
    OwnSelection( tripleValue, ( XtPointer )FALSE, ( XtPointer )TRUE );

    rgb_values.r = colors[MIX_COLOR].red;
    rgb_values.g = colors[MIX_COLOR].green;
    rgb_values.b = colors[MIX_COLOR].blue;

    if( workingHSV ) {
        double posx, posy;
        double cos_theta, cos_phi, rtheta, theta, phi;
        double length_a, length_b, a_dot_b;
    
        hsv_values = RGBToHSV( rgb_values );
    
        /* reverse the calculation of hue saturation to draw the box */
        theta = hsv_values.h * 360.;
        rtheta = DEGRAD( theta );
        cos_theta = cos( rtheta );
        posx = cos_theta * hsv_values.s;
        xpos = ( int )( ( posx + 1. ) * 100. );
    
        phi = theta - 45.;
        rtheta = DEGRAD( phi );
        cos_phi = cos( rtheta );
        posy = ( cos_phi * 2. * hsv_values.s ) / 1.414213 - posx;
        ypos = ( int )( ( posy + 1. ) * 100. );
        ypos = 200 - ypos;
    
        hsv_Expose( hsvColor );
    
        XawScrollbarSetThumb( hueScroll, ( float )( 1.0 - hsv_values.h ),
                            ( float )0.025 );
        ChangeLabel( HUE, ( float )( 1.0 - hsv_values.h ) );
        XawScrollbarSetThumb( satScroll, ( float )( 1.0 - hsv_values.s ),
                            ( float )0.025 );
        ChangeLabel( SAT, ( float )( 1.0 - hsv_values.s ) );
        XawScrollbarSetThumb( valScroll, ( float )( 1.0 - hsv_values.v ),
                            ( float )0.025 );
        ChangeLabel( VAL, ( float )( 1.0 - hsv_values.v ) );

        if( depth != 8 ) {
            extern Window                mixed_window;

            if( mixed_rd != (PEXRenderer)0 ) {
                mixed_rdVals.background_color.type = PEXColorTypeRGB;
                mixed_rdVals.background_color.value.rgb.red = 
                    (float)rgb_values.r / 65536.0;
                mixed_rdVals.background_color.value.rgb.green =
                    (float)rgb_values.g / 65536.0;
                mixed_rdVals.background_color.value.rgb.blue =
                    (float)rgb_values.b / 65536.0;
                PEXChangeRenderer( display, mixed_rd,
                    PEXRABackgroundColor, &mixed_rdVals );
                PEXBeginRendering( display, mixed_window, mixed_rd );
                PEXEndRendering( display, mixed_rd, False );
            }
        }
    }
    else if( workingHLS ) {
        double posx, posy;
        double cos_theta, cos_phi, rtheta, theta, phi;
        double length_a, length_b, a_dot_b;

        hls_values = RGBToHLS( rgb_values );
    
        /* reverse the calculation of hue saturation to draw the box */
        theta = hls_values.h * 360.;
        rtheta = DEGRAD( theta );
        cos_theta = cos( rtheta );
        posx = cos_theta * hls_values.s;
        xpos = ( int )( ( posx + 1. ) * 100. );

        phi = theta - 45.;
        rtheta = DEGRAD( phi );
        cos_phi = cos( rtheta );
        posy = ( cos_phi * 2. * hls_values.s ) / 1.414213 - posx;
        ypos = ( int )( ( posy + 1. ) * 100. );
        ypos = 200 - ypos;

        hls_Expose( hlsColor );

        XawScrollbarSetThumb( hueScroll, ( float )( 1.0 - hls_values.h ),
                            ( float )0.025 );
        ChangeLabel( HUE, ( float )( 1.0 - hls_values.h ) );
        XawScrollbarSetThumb( lgtScroll, ( float )( 1.0 - hls_values.l ),
                            ( float )0.025 );
        ChangeLabel( LGT, ( float )( 1.0 - hls_values.l ) );
        XawScrollbarSetThumb( satScroll, ( float )( 1.0 - hls_values.s ),
                            ( float )0.025 );
        ChangeLabel( SAT, ( float )( 1.0 - hls_values.s ) );

        if( depth != 8 ) {
            extern Window                mixed_window;
    
            if( mixed_rd != (PEXRenderer)0 ) {
                mixed_rdVals.background_color.type = PEXColorTypeRGB;
                mixed_rdVals.background_color.value.rgb.red =
                    (float)rgb_values.r / 65536.0;
                mixed_rdVals.background_color.value.rgb.green =
                    (float)rgb_values.g / 65536.0;
                mixed_rdVals.background_color.value.rgb.blue =
                    (float)rgb_values.b / 65536.0;
                PEXChangeRenderer( display, mixed_rd,
                    PEXRABackgroundColor, &mixed_rdVals );
                PEXBeginRendering( display, mixed_window, mixed_rd );
                PEXEndRendering( display, mixed_rd, False );
            }
        }
    }
    else if( workingCIE ) {

        cie_values = RGBToCIE( rgb_values );

        xpos = ( cie_values.X /
               ( cie_values.X + cie_values.Y + cie_values.Z ) ) * 200. + 0.5;
        ypos = ( cie_values.Y /
               ( cie_values.X + cie_values.Y + cie_values.Z ) ) * 200. + 0.5;
    
        cie_Expose( cieColor );

        XawScrollbarSetThumb( XScroll, cie_values.X, ( float )0.025 );
        ChangeLabel( XVAL, cie_values.X );
        XawScrollbarSetThumb( YScroll, cie_values.Y, ( float )0.025 );
        ChangeLabel( YVAL, cie_values.Y );
        XawScrollbarSetThumb( ZScroll, cie_values.Z, ( float )0.025 );
        ChangeLabel( ZVAL, cie_values.Z );

        cie_values.X *= 255.;
        cie_values.Y *= 255.;
        cie_values.Z *= 255.;

        if( depth != 8 ) {
            extern Window                mixed_window;

            if( mixed_rd != (PEXRenderer)0 ) {
                mixed_rdVals.background_color.type = PEXColorTypeRGB;
                mixed_rdVals.background_color.value.rgb.red =
                    (float)rgb_values.r / 65536.0;
                mixed_rdVals.background_color.value.rgb.green =
                    (float)rgb_values.g / 65536.0;
                mixed_rdVals.background_color.value.rgb.blue =
                    (float)rgb_values.b / 65536.0;
                PEXChangeRenderer( display, mixed_rd,
                    PEXRABackgroundColor, &mixed_rdVals );
                PEXBeginRendering( display, mixed_window, mixed_rd );
                PEXEndRendering( display, mixed_rd, False );
            }
        }
    }
}


XtActionProc
hsv_Enter( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    XInstallColormap( display, colormap );
}


XtActionProc
hsv_Configure( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
}


XtActionProc
hsv_Expose( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    extern Window window;
    extern int    depth;
    extern GC     gc;

    if( (xpos <= 0) || (xpos >= 200) || (ypos <= 0) || (ypos >= 200) ) return;

    if( depth == 8 ) {
        extern Pixmap pixmap;

        if( (window != (Window)0) && (pixmap != (Pixmap)0) ) {
            XCopyArea( display, pixmap, window, gc, 0, 0, 200, 200, 0, 0 );
            XDrawRectangle( display, window, gc, xpos-3, ypos-3, 6, 7 );
        }
    }
    else {
        static float mark[10];
        extern XID resource_id;
        extern PEXOCRequestType dest;
        extern PEXRenderer  rd;
        extern PEXStructure sid;

        if( (rd != (PEXRenderer)0) && (sid != (PEXStructure)0) ) {
            mark[0] = ( xpos-3 )/200.;  mark[1] = 1. - ( ypos-3 )/200.;
            mark[2] = mark[0];          mark[3] = 1. - ( ypos+4 )/200.;
            mark[4] = ( xpos+3 )/200.;  mark[5] = mark[3];
            mark[6] = mark[4];          mark[7] = mark[1];
            mark[8] = mark[0];          mark[9] = mark[1];

            PEXSetElementPtr( display, sid, PEXBeginning, 0 );
            PEXSetElementPtrAtLabel( display, sid, 1, 1 );
            PEXDeleteElements( display, sid, PEXCurrent, 0, PEXEnd, 0 );
            PEXPolyline2D( display, resource_id, dest, 5, (PEXCoord2D *)mark );

            PEXRenderNetwork( display, window, rd, sid );
            XFlush( display );
        }
    }
}


XtActionProc
hsv_BtnDn( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    double posx, posy;
    double cos_theta, rtheta, theta;
    double length_a, length_b, a_dot_b;
    float  saturation, hue;

    xpos = ( ( XButtonEvent * )event )->x;
    ypos = ( ( XButtonEvent * )event )->y;
    ypos = 200 - ypos;
    posx = ( double )xpos * 0.01 - 1.;
    posy = ( double )ypos * 0.01 - 1.;
    saturation = ( float )sqrt( posx * posx + posy * posy );
    if( saturation > 1.0 ) return;
    length_a = 1.;
    length_b = saturation;
    a_dot_b = posx;
    /* cos_theta = a_dot_b / ( length_a * length_b ); */
    cos_theta = posx / ( double )saturation;
    rtheta = acos( cos_theta );
    theta = RADDEG( rtheta );
    if( ypos <= 100 ) theta = 360. - theta;
    hue = ( float )( theta / 360. );

    saturation = 1. - saturation;
    hue = 1. - hue;
    Update_HSV( hueScroll, HUE, ( XtPointer )( &hue ) );
    Update_HSV( satScroll, SAT, ( XtPointer )( &saturation ) );
}


XtActionProc
hls_Enter( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    XInstallColormap( display, colormap );
}


XtActionProc
hls_Configure( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
}


XtActionProc
hls_Expose( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    extern Window window;
    extern int    depth;
    extern GC     gc;

    if( (xpos <= 0) || (xpos >= 200) || (ypos <= 0) || (ypos >= 200) ) return;

    if( depth == 8 ) {
        extern Pixmap pixmap;

        if( (window != (Window)0) && (pixmap != (Pixmap)0) ) {
            XCopyArea( display, pixmap, window, gc, 0, 0, 200, 200, 0, 0 );
            XDrawRectangle( display, window, gc, xpos-3, ypos-3, 6, 7 );
        }
    }
    else {
        static float mark[10];
        extern XID resource_id;
        extern PEXOCRequestType dest;
        extern PEXRenderer  rd;
        extern PEXStructure sid;

        if( (rd != (PEXRenderer)0) && (sid != (PEXStructure)0) ) {
            mark[0] = ( xpos-3 )/200.;  mark[1] = 1. - ( ypos-3 )/200.;
            mark[2] = mark[0];          mark[3] = 1. - ( ypos+4 )/200.;
            mark[4] = ( xpos+3 )/200.;  mark[5] = mark[3];
            mark[6] = mark[4];          mark[7] = mark[1];
            mark[8] = mark[0];          mark[9] = mark[1];

            PEXSetElementPtr( display, sid, PEXBeginning, 0 );
            PEXSetElementPtrAtLabel( display, sid, 1, 1 );
            PEXDeleteElements( display, sid, PEXCurrent, 0, PEXEnd, 0 );
            PEXPolyline2D( display, resource_id, dest, 5, (PEXCoord2D *)mark );

            PEXRenderNetwork( display, window, rd, sid );
            XFlush( display );
        }
    }
}


XtActionProc
hls_BtnDn( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    int xpos, ypos;
    double posx, posy;
    double cos_theta, rtheta, theta;
    double length_a, length_b, a_dot_b;
    float  saturation, hue;

    xpos = ( ( XButtonEvent * )event )->x;
    ypos = ( ( XButtonEvent * )event )->y;
    ypos = 200 - ypos;
    posx = ( double )xpos * 0.01 - 1.;
    posy = ( double )ypos * 0.01 - 1.;
    saturation = ( float )sqrt( posx * posx + posy * posy );
    if( saturation > 1.0 ) return;
    length_a = 1.;
    length_b = saturation;
    a_dot_b = posx;
    /* cos_theta = a_dot_b / ( length_a * length_b ); */
    cos_theta = posx / ( double )saturation;
    rtheta = acos( cos_theta );
    theta = RADDEG( rtheta );
    if( ypos <= 100 ) theta = 360. - theta;
    hue = ( float )( theta / 360. );

    saturation = 1. - saturation;
    hue = 1. - hue;
    Update_HLS( hueScroll, HUE, ( XtPointer )( &hue ) );
    Update_HLS( satScroll, SAT, ( XtPointer )( &saturation ) );
}


XtActionProc
cie_Enter( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    XInstallColormap( display, colormap );
}


XtActionProc
cie_Configure( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
}


XtActionProc
cie_Expose( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    extern int          depth;
    extern Window       window;
    char *string;

    if( (xpos <= 0) || (xpos >= 200) || (ypos <= 0) || (ypos >= 200) ) return;

    if( depth == 8 ) {
        extern Pixmap   pixmap;
        extern Colormap colormap;
        extern GC       gc;

        if( (window != (Window)0) && (pixmap != (Pixmap)0) ) {
            XCopyArea( display, pixmap, window, gc, 0, 0, 200, 200, 0, 0 );
            XSetForeground( display, gc, AllocColor(display,colormap,"black") );
            XDrawRectangle( display, window, gc, xpos-3, 200-ypos-3, 6, 7 );
            string = ( char * ) malloc( 20 );
            sprintf( string, "%.2f %.2f", xpos/200., ypos/200. );
            XSetForeground( display, gc, AllocColor(display,colormap,"white") );
            XDrawString( display, window, gc, 90, 42, string, strlen(string) );
        }
    }
    else {
        static float mark[10];
        PEXColor   t_color;
        PEXCoord2D origin;
        PEXEncodedTextData encodedTextData;

        extern XID resource_id;
        extern PEXOCRequestType dest;
        extern PEXRenderer  rd;
        extern PEXStructure sid;

        if( (rd != (PEXRenderer)0) && (sid != (PEXStructure)0) ) {
            mark[0] = ( xpos-3 )/200.;  mark[1] = ( ypos-3 )/200.;
            mark[2] = mark[0];          mark[3] = ( ypos+4 )/200.;
            mark[4] = ( xpos+3 )/200.;  mark[5] = mark[3];
            mark[6] = mark[4];          mark[7] = mark[1];
            mark[8] = mark[0];          mark[9] = mark[1];
    
            PEXSetElementPtr( display, sid, PEXBeginning, 0 );
            PEXSetElementPtrAtLabel( display, sid, 1, 1 );
            PEXDeleteElements( display, sid, PEXCurrent, 0, PEXEnd, 0 );
            PEXPolyline2D( display, resource_id, dest, 5, (PEXCoord2D *)mark );
            PEXSetTextColorIndex( display, resource_id, dest, 23 );
            string = ( char * ) malloc( 20 );
            sprintf( string, "%.2f %.2f", xpos/200., ypos/200. );
            PEXSetCharHeight( display, resource_id, dest, .055 );
            PEXSetCharExpansion( display, resource_id, dest, .9 );
            origin.x = 0.45;  origin.y = 0.8;
            PEXText2D( display, resource_id, dest, &origin, 
                strlen( string ), string );
    
            PEXRenderNetwork( display, window, rd, sid );
            XFlush( display );
        }
    }
}


void
cie_BtnDn( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    float x, y, z;
    float X, Y, Z;
    float Ylow, Yhigh;
    RGB rgb_values;
    RGBf rgbf_values;
    Boolean found = False;
    char *string;
    char hexvalue[10];

    xpos = ( ( XButtonEvent * )event )->x;
    ypos = ( ( XButtonEvent * )event )->y;
    ypos = 200 - ypos;
    x = xpos / 200.;
    y = ypos / 200.;

    Ylow = 0.;
    Yhigh = 512.;
    while( !found ) {
        Y = Ylow + ( Yhigh - Ylow ) / 2.;
        cie_values.X = x/y * Y;
        cie_values.Y = Y;
        cie_values.Z = ( 1-x-y )/y * Y;
        rgbf_values = CIEToRGBs( cie_values );
        if( ( rgbf_values.r >= 0 ) && ( rgbf_values.r <= 255. ) &&
            ( rgbf_values.g >= 0 ) && ( rgbf_values.g <= 255. ) &&
            ( rgbf_values.b >= 0 ) && ( rgbf_values.b <= 255. ) ) {
            Ylow = Y;
        }
        else
            Yhigh = Y;
        if( ( Yhigh - Ylow ) < 0.0001 ) found = True;
    }

    if( Y > 255. ) Y = 255.;
    X = x/y * Y;
    Z = ( 1-x-y )/y * Y;
    X /= 255.; Y /= 255.; Z /= 255.;

    Update_CIE( XScroll, XVAL, ( XtPointer)( &X ) );
    Update_CIE( YScroll, YVAL, ( XtPointer)( &Y ) );
    Update_CIE( ZScroll, ZVAL, ( XtPointer)( &Z ) );
}


void
set_scroll( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    int button_down = WhichButton( params[0] );
    last_pos = event->xbutton.x;
    buttons_down |= button_down;
    ChangeBorder( button_down, FALSE );
}


void
stop_scroll( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    int button_up = WhichButton( params[0] );
    buttons_down &= ~button_up;
    ChangeBorder( button_up, TRUE );
    if ( !buttons_down )
        update_triple( w, event, params, num_params );
}


void
move_scroll( w, event, params, num_params )
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
#define ADJUST_CHANGE( color ) if ( change < 0 ) { \
    if ( color + change < 0 ) \
        change = -color; \
    } else { \
        if ( color + change > 255 ) \
            change = 255-color; \
    }

    int change;
    float pass_value;
    int red_pos, green_pos, blue_pos;

    if ( buttons_down == 0 )
        return;

    change = last_pos - event->xmotion.x;
    last_pos = event->xmotion.x;

    if ( buttons_down & RED ) {
        red_pos = colors[MIX_COLOR].red/256;
        ADJUST_CHANGE( red_pos );
    }

    if ( buttons_down & GREEN ) {
        green_pos = colors[MIX_COLOR].green/256;
        ADJUST_CHANGE( green_pos );
    }

    if ( buttons_down & BLUE ) {
        blue_pos = colors[MIX_COLOR].blue/256;
        ADJUST_CHANGE( blue_pos );
    }

    red_pos += change;
    green_pos += change;
    blue_pos += change;

    /* update the new scroll bar positions and change the color */
    do_change = FALSE;

    if ( buttons_down & RED )    {
        pass_value = 1.0 - ( float )red_pos/255;
        Thumbed( redScroll, RED, ( XtPointer )( &pass_value ) );
    }

    if ( buttons_down & GREEN )    {
        pass_value = 1.0 - ( float )green_pos/255;
        Thumbed( greenScroll, GREEN, ( XtPointer )( &pass_value ) );
    }

    if ( buttons_down & BLUE )    {
        pass_value = 1.0 - ( float )blue_pos/255;
        Thumbed( blueScroll, BLUE, ( XtPointer )( &pass_value ) );
    }

    do_change = TRUE;
    XStoreColors( display, cmap, colors, 4 );
    update_triple( ( Widget )NULL, ( XEvent * )NULL,
        ( String * )NULL, ( Cardinal * )NULL );
}


void
Scrolled( w, closure, change )
Widget w;
XtPointer closure, change;
{
    Boolean going_up = ( int )change < 0;
    int which = ( int )closure;
    int pos = 0;

    switch ( which ) {
    case RED:
        pos = COLOR( MIX_COLOR,red );
        break;
    case BLUE:
        pos = COLOR( MIX_COLOR,blue );
        break;
    case GREEN:
        pos = COLOR( MIX_COLOR,green );
        break;
    case LOCKED:
        pos = 255 - ( int )( locked_top * 255 + 0.5 );
        break;
    case HUE:
    case SAT:
    case VAL:
        /* Not yet implemented */
        return;
    case XVAL:
    case YVAL:
    case ZVAL:
        /* Not yet implemented */
        return;
    default:
        fprintf( stderr, "Oops Scroll calldata invalid\n" );
        exit( 1 );
    }

    if ( going_up ) {
        if ( pos > 0 )
            pos--;
    } else {
        if ( pos < 255 )
            pos++;
    }

    pass_value = 1.0 - ( double )pos/255;
    Thumbed( w, closure, ( XtPointer )( &pass_value ) );
}



void
Update_HSV( w, closure, ptr )
Widget w;
XtPointer closure, ptr;
{
    int which = ( int )closure;
    float per = *( float* )ptr;
    double top = ( double )per;
    char hexvalue[10];
    extern int depth;

    double posx, posy;
    double cos_theta, cos_phi, rtheta, theta, phi;
    double length_a, length_b, a_dot_b;
    float  saturation, hue;

    ChangeLabel( which, top );
    switch ( which ) {
    case HUE:
        hsv_values.h = 1. - top;
        break;
    case SAT:
        hsv_values.s = 1. - top;
        break;
    case VAL:
        hsv_values.v = 1. - top;
        break;
    }

    /* reverse the calculation of hue saturation to draw the box */
    theta = hsv_values.h * 360.;
    rtheta = DEGRAD( theta );
    cos_theta = cos( rtheta );
    posx = cos_theta * hsv_values.s;
    xpos = ( int )( ( posx + 1. ) * 100. );

    phi = theta - 45.;
    rtheta = DEGRAD( phi );
    cos_phi = cos( rtheta );
    posy = ( cos_phi * 2. * hsv_values.s ) / 1.414213 - posx;
    ypos = ( int )( ( posy + 1. ) * 100. );
    ypos = 200 - ypos;

    hsv_Expose( hsvColor );
    rgb_values = HSVToRGB( hsv_values );

    colors[MIX_COLOR].red = rgb_values.r;
    colors[MIX_COLOR].green = rgb_values.g;
    colors[MIX_COLOR].blue = rgb_values.b;

    sprintf( hexvalue, prog_res.format,
                  COLOR( MIX_COLOR,red ), COLOR( MIX_COLOR,green ),
                  COLOR( MIX_COLOR,blue ) );
    XtVaSetValues( tripleValue, XtNlabel, hexvalue, NULL );
    OwnSelection( tripleValue, ( XtPointer )FALSE, ( XtPointer )TRUE );

    XawScrollbarSetThumb( w, top, ( float )0.025 );

    if( depth == 8 )
        XtVaSetValues(mixedColor, XtNbackground, colors[MIX_COLOR].pixel, NULL);
    else {
        extern Window mixed_window;

        if( mixed_rd != (PEXRenderer)0 ) {
            mixed_rdVals.background_color.type = PEXColorTypeRGB;
            mixed_rdVals.background_color.value.rgb.red =
                (float)rgb_values.r / 65536.0;
            mixed_rdVals.background_color.value.rgb.green =
                (float)rgb_values.g / 65536.0;
            mixed_rdVals.background_color.value.rgb.blue =
                (float)rgb_values.b / 65536.0;
            PEXChangeRenderer( display, mixed_rd,
                PEXRABackgroundColor, &mixed_rdVals );
            PEXBeginRendering( display, mixed_window, mixed_rd );
            PEXEndRendering( display, mixed_rd, False );
        }
    }

    XStoreColors( display, cmap, colors, 4 );

    do_change = FALSE;
    pass_value = 1.0 - (float)rgb_values.r / 65536.0;
    Thumbed( redScroll, ( XtPointer )RED, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.g / 65536.0;
    Thumbed( greenScroll, ( XtPointer )GREEN, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.b / 65536.0;
    Thumbed( blueScroll, ( XtPointer )BLUE, ( XtPointer )( &pass_value ) );
    do_change = TRUE;
}


void
Update_HLS( w, closure, ptr )
Widget w;
XtPointer closure, ptr;
{
    int which = ( int )closure;
    float per = *( float* )ptr;
    double top = ( double )per;
    char hexvalue[10];
    extern int depth;

    double posx, posy;
    double cos_theta, cos_phi, rtheta, theta, phi;
    double length_a, length_b, a_dot_b;
    float  saturation, hue;

    ChangeLabel( which, top );
    switch ( which ) {
    case HUE:
        hls_values.h = 1. - top;
        break;
    case LGT:
        hls_values.l = 1. - top;
        break;
    case SAT:
        hls_values.s = 1. - top;
        break;
    }

    /* reverse the calculation of hue saturation to draw the box */
    theta = hls_values.h * 360.;
    rtheta = DEGRAD( theta );
    cos_theta = cos( rtheta );
    posx = cos_theta * hls_values.s;
    xpos = ( int )( ( posx + 1. ) * 100. );

    phi = theta - 45.;
    rtheta = DEGRAD( phi );
    cos_phi = cos( rtheta );
    posy = ( cos_phi * 2. * hls_values.s ) / 1.414213 - posx;
    ypos = ( int )( ( posy + 1. ) * 100. );
    ypos = 200 - ypos;

    hls_Expose( hlsColor );
    rgb_values = HLSToRGB( hls_values );

    colors[MIX_COLOR].red = rgb_values.r;
    colors[MIX_COLOR].green = rgb_values.g;
    colors[MIX_COLOR].blue = rgb_values.b;

    sprintf( hexvalue, prog_res.format, COLOR( MIX_COLOR, red ),
        COLOR( MIX_COLOR, green ), COLOR( MIX_COLOR, blue ) );
    XtVaSetValues( tripleValue, XtNlabel, hexvalue, NULL );
    OwnSelection( tripleValue, ( XtPointer )FALSE, ( XtPointer )TRUE );

    XawScrollbarSetThumb( w, top, ( float )0.025 );

    if( depth == 8 )
        XtVaSetValues(mixedColor, XtNbackground, colors[MIX_COLOR].pixel, NULL);
    else {
        extern Window mixed_window;

        if( mixed_rd != (PEXRenderer)0 ) {
            mixed_rdVals.background_color.type = PEXColorTypeRGB;
            mixed_rdVals.background_color.value.rgb.red =
                (float)rgb_values.r / 65536.0;
            mixed_rdVals.background_color.value.rgb.green =
                (float)rgb_values.g / 65536.0;
            mixed_rdVals.background_color.value.rgb.blue =
                (float)rgb_values.b / 65536.0;
            PEXChangeRenderer( display, mixed_rd,
                PEXRABackgroundColor, &mixed_rdVals );
            PEXBeginRendering( display, mixed_window, mixed_rd );
            PEXEndRendering( display, mixed_rd, False );
        }
    }

    XStoreColors( display, cmap, colors, 4 );

    do_change = FALSE;
    pass_value = 1.0 - (float)rgb_values.r / 65536.0;
    Thumbed( redScroll, ( XtPointer )RED, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.g / 65536.0;
    Thumbed( greenScroll, ( XtPointer )GREEN, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.b / 65536.0;
    Thumbed( blueScroll, ( XtPointer )BLUE, ( XtPointer )( &pass_value ) );
    do_change = TRUE;
}


void
Update_CIE( w, closure, ptr )
Widget w;
XtPointer closure, ptr;
{
    int which = ( int )closure;
    float per = *( float* )ptr;
    double top = ( double )per * 255.;
    char hexvalue[10];
    extern int depth;

    float x, y;

    ChangeLabel( which, top/255. );
    switch ( which ) {
    case XVAL:
        cie_values.X = top;
        break;
    case YVAL:
        cie_values.Y = top;
        break;
    case ZVAL:
        cie_values.Z = top;
        break;
    }

    xpos = ( cie_values.X /
           ( cie_values.X + cie_values.Y + cie_values.Z ) ) * 200. + 0.5;
    ypos = ( cie_values.Y /
           ( cie_values.X + cie_values.Y + cie_values.Z ) ) * 200. + 0.5;

    cie_Expose( cieColor );
    rgb_values = CIEToRGB( cie_values );

    colors[MIX_COLOR].red = rgb_values.r;
    colors[MIX_COLOR].green = rgb_values.g;
    colors[MIX_COLOR].blue = rgb_values.b;

    sprintf( hexvalue, prog_res.format,
                  COLOR( MIX_COLOR,red ), COLOR( MIX_COLOR,green ),
                  COLOR( MIX_COLOR,blue ) );
    XtVaSetValues( tripleValue, XtNlabel, hexvalue, NULL );
    OwnSelection( tripleValue, ( XtPointer )FALSE, ( XtPointer )TRUE );

    XawScrollbarSetThumb( XScroll, cie_values.X/255., ( float )0.025 );
    XawScrollbarSetThumb( YScroll, cie_values.Y/255., ( float )0.025 );
    XawScrollbarSetThumb( ZScroll, cie_values.Z/255., ( float )0.025 );

    if( depth == 8 )
        XtVaSetValues(mixedColor, XtNbackground, colors[MIX_COLOR].pixel, NULL);
    else {
        extern Window mixed_window;

        if( mixed_rd != (PEXRenderer)0 ) {
            mixed_rdVals.background_color.type = PEXColorTypeRGB;
            mixed_rdVals.background_color.value.rgb.red =
                (float)rgb_values.r / 65536.0;
            mixed_rdVals.background_color.value.rgb.green =
                (float)rgb_values.g / 65536.0;
            mixed_rdVals.background_color.value.rgb.blue =
                (float)rgb_values.b / 65536.0;
            PEXChangeRenderer( display, mixed_rd,
                PEXRABackgroundColor, &mixed_rdVals );
            PEXBeginRendering( display, mixed_window, mixed_rd );
            PEXEndRendering( display, mixed_rd, False );
        }
    }

    XStoreColors( display, cmap, colors, 4 );

    do_change = FALSE;
    pass_value = 1.0 - (float)rgb_values.r / 65536.0;
    Thumbed( redScroll, ( XtPointer )RED, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.g / 65536.0;
    Thumbed( greenScroll, ( XtPointer )GREEN, ( XtPointer )( &pass_value ) );
    pass_value = 1.0 - (float)rgb_values.b / 65536.0;
    Thumbed( blueScroll, ( XtPointer )BLUE, ( XtPointer )( &pass_value ) );
    do_change = TRUE;
}


void
Thumbed( w, closure, ptr )
Widget w;
XtPointer closure, ptr;
{
    int which = ( int )closure;
    int mix;
    float per = *( float* )ptr;
    double top = ( double )per;
    XEvent event;
    char hexvalue[10];
    
    sprintf( hexvalue, prog_res.format,
                  COLOR( MIX_COLOR,red ), COLOR( MIX_COLOR,green ),
                  COLOR( MIX_COLOR,blue ) );
    XtVaSetValues( tripleValue, XtNlabel, hexvalue, NULL );
    OwnSelection( tripleValue, ( XtPointer )FALSE, ( XtPointer )TRUE );

    mix = ( int ) ( ( 1.0 - top ) * 256.0 * 256.0 );
    if ( mix > 0xFFFF )
        mix = 0xFFFF;

    ChangeBorder( which );
    ChangeLabel( which,top );
    switch ( which ) {
    case RED:
        colors[MIX_COLOR].red = colors[RED_COLOR].red = mix;
        red_top = top;
        rgbf_values.r = 1. - top;
        break;
    case GREEN:
        colors[MIX_COLOR].green = colors[GREEN_COLOR].green = mix;
        green_top = top;
        rgbf_values.g = 1. - top;
        break;
    case BLUE:
        colors[MIX_COLOR].blue = colors[BLUE_COLOR].blue = mix;
        blue_top = top;
        rgbf_values.b = 1. - top;
        break;
    case LOCKED:
        buttons_down = bars_locked;
        last_pos = ( int )( ( double )locked_top*255 );
        event.xmotion.x = ( int )( ( double )top*255 );
        move_scroll( w, &event, ( String * )NULL, ( Cardinal * )NULL );
        buttons_down = 0;
        return;
    }
    if ( do_change ) {
        XStoreColors( display, cmap, colors, 4 );
        update_triple( ( Widget )NULL, ( XEvent * )NULL,
            ( String * )NULL, ( Cardinal * )NULL );
    }
    XawScrollbarSetThumb( w, top, ( float )0.025 );

    move_lock(  );
}


move_lock(  )
{
    locked_top = 1.0;
    if ( bars_locked & RED )
        locked_top = MIN( locked_top, red_top );
    if ( bars_locked & BLUE )
        locked_top = MIN( locked_top, blue_top );
    if ( bars_locked & GREEN )
        locked_top = MIN( locked_top, green_top );
    XawScrollbarSetThumb( lockedScroll, locked_top, ( float )0.025 );
}


/* Set the current selection of the PRIMARY_COLOR property to this colour */
/* ARGSUSED */
void
Set_Selection( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    Boolean own;

    XtVaGetValues( w, XtNstate, &own, NULL );
    OwnSelection( w, ( XtPointer )TRUE, ( XtPointer )( own ? TRUE : FALSE ) );
}


Boolean
ConvertSelection( w, selection, target, type, value, length, format )
Widget w;
Atom *selection, *target, *type;
XtPointer *value;
unsigned long *length;
int *format;
{
    if ( XmuConvertStandardSelection( w, selection, target, type, value,
        length, format ) )
        return TRUE;

    if ( *target == XA_STRING ) {
        String color_str;

        XtVaGetValues( tripleValue, XtNlabel, &color_str, NULL );
        *type = XA_STRING;
        *value = color_str;
        *length = strlen( *value );
        *format = 8;
        return ( TRUE );
    }
    return ( FALSE );
}


/* ARGSUSED */
void
LoseSelection( w, selection )
Widget w;
Atom *selection;
{
    XtVaSetValues( w, XtNstate, FALSE, NULL );
}


/* ARGSUSED */
void
DoneSelection( w, selection, target )
Widget w;
Atom *selection, *target;
{
    /* we don't need to do anything here */
}


/* ARGSUSED */
void
OwnSelection( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    Time time = XtLastTimestampProcessed( XtDisplay( w ) );
    Boolean primary = ( Boolean )client_data;
    Boolean own = ( Boolean )call_data;

    if ( _XA_PRIMARY_COLOR == NULL )
        _XA_PRIMARY_COLOR = XmuMakeAtom( PRIMARY_COLOR );

    if ( own ) {
        XtOwnSelection( w, XA_PRIMARY_COLOR, time,
            ConvertSelection, LoseSelection, DoneSelection );
        if ( primary )
            XtOwnSelection( w, XA_PRIMARY, time,
                ConvertSelection, LoseSelection, DoneSelection );
    } else {
        XtDisownSelection( w, XA_PRIMARY_COLOR, time );
        if ( primary )
            XtDisownSelection( w, XA_PRIMARY, time );
    }
}
