
#include "unix.h"
#include "x.h"
#include "xt.h"
#include "xpex.h"
#include "xmu.h"
#include "color.h"
#include "externs.h"
#include "globals.h"

extern XtAppContext app_context;
extern Display     *display;
extern GC           gc;
extern Visual      *visual;
extern int          depth;
extern Colormap     colormap;
       XStandardColormap   cmap_info;
       PEXColorApproxEntry approxEnt;
extern Colormap     cmap;
       Cursor       cursor;
extern XColor       colors[64];
extern GC  boxedGC, boxedChangedGC, unboxedGC;

extern int bars_locked;
extern Boolean do_change;
extern Pixel original_background;
extern float pass_value;

extern Widget toplevel, mixingForm, mixedColor;
extern Widget rgb, hsv, hls, cie;

Widget cieColor;

extern Widget title;
extern Widget tripleValue;
extern Widget XLocked, YLocked, ZLocked;
extern Widget XScroll, YScroll, ZScroll;
extern Widget XLabel, YLabel, ZLabel;
extern Widget quit;

extern Widget redLocked, greenLocked, blueLocked;
extern Widget redLabel, greenLabel, blueLabel;
extern Widget redScroll, greenScroll, blueScroll, lockedScroll;

extern Boolean workingRGB, workingHSV, workingHLS, workingCIE;

extern XtActionProc cie_BtnDn(), cie_Configure(), cie_Expose();

XtActionsRec cie_actions[] =
{
   { "BtnDn",     (XtActionProc)cie_BtnDn },
   { "Configure", (XtActionProc)cie_Configure },
   { "Expose",    (XtActionProc)cie_Expose }
};


String cie_translations =
  "<BtnDown>:     BtnDn()     \n\
   <Configure>:   Configure() \n\
   <Expose>:      Expose() ";


void makeCIE_Color( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    Widget popup;
    XGCValues values;
    extern PEXColorSpecifier colorSpec;
    int type;
    XVisualInfo       visualInfo;
    PEXExtensionInfo *pexinfo;
    char              err[80];

    /* See if a PEX extension exists and initialize PEXlib. */
    if ( PEXInitialize( display, &pexinfo, 80, err ) != 0 ) {
        fprintf( stderr, "No PEX extension.\n" );
        fprintf( stderr, "%s\n", err );
        exit(1);
    }

#ifndef PSEUDO
    /* Determine the best visual to use. */
    type = ora_find_best_visual( display, &visualInfo );
    /* If type == PEXTrue after this statement */
    /* then pixmap drawable is available.   */
    depth = visualInfo.depth;
    visual = visualInfo.visual;
#else
    depth = DefaultDepth( display, DefaultScreen( display ) );
    visual = DefaultVisual( display, DefaultScreen( display ) );
#endif

    if( depth != 8 )
        ora_create_colormap( display, &visualInfo, &cmap_info, &approxEnt );
    else {
        /* Get a standard colormap for the visual. */
        if ( !ora_get_standard_colormap( display, &visualInfo, &cmap_info ) ) {
            fprintf ( stderr, "Cannot find a standard colormap\n" );
            exit(1);
        }
        ora_set_stdcmap_approx( &visualInfo, &cmap_info, &approxEnt );
    }

    colormap = cmap_info.colormap;

    XtSetSensitive( rgb, False );    workingRGB = False;
    XtSetSensitive( hsv, False );    workingHSV = False;
    XtSetSensitive( hls, False );    workingHLS = False;
    XtSetSensitive( cie, False );    workingCIE = True;
                                     colorSpec.type = PEXColorTypeCIE;

    popup = XtVaCreateManagedWidget( "ColorEditor",
        topLevelShellWidgetClass, toplevel,
        XtNwidth, 425,
        XtNheight, 353,
        XtNvisual, visual,
        XtNdepth, depth,
        XtNcolormap, colormap,
        NULL);
    mixingForm = XtVaCreateManagedWidget("mixingForm", formWidgetClass, popup,
        NULL);

    title = XtVaCreateManagedWidget("title", labelWidgetClass, mixingForm,
        XtNlabel, "Choose CIEXYZ Color",
        NULL);

    cursor = XCreateFontCursor( display, XC_dotbox );
    cieColor = XtVaCreateManagedWidget("cieColor", labelWidgetClass,
        mixingForm,
        XtNfromVert, title,
        XtNheight, 200,
        XtNwidth, 200,
        XtNlabel, "",
        XtNcursor, cursor,
        NULL);
    mixedColor = XtVaCreateManagedWidget("mixedColor", labelWidgetClass,
        mixingForm,
        XtNlabel, "",
        XtNheight, 200,
        XtNwidth, 90,
        XtNbackground, colors[MIX_COLOR].pixel,
        XtNfromVert, title,
        XtNfromHoriz, cieColor,
        NULL);

    tripleValue = XtVaCreateManagedWidget("tripleValue", toggleWidgetClass,
        mixingForm,
        XtNfromHoriz, cieColor,
        XtNwidth, 90,
        XtNheight, 23,
        NULL);

    XLocked = XtVaCreateManagedWidget("XLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, cieColor,
        NULL);
    XScroll = XtVaCreateManagedWidget("XScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, cieColor,
        XtNfromHoriz, XLocked,
        NULL);
    XLabel = XtVaCreateManagedWidget("XLabel", labelWidgetClass, mixingForm,
        XtNfromVert, cieColor,
        XtNfromHoriz, XScroll,
        NULL);

    YLocked = XtVaCreateManagedWidget("YLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, XLocked,
        NULL);
    YScroll = XtVaCreateManagedWidget("YScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, XLocked,
        XtNfromHoriz, YLocked,
        NULL);
    YLabel = XtVaCreateManagedWidget("YLabel", labelWidgetClass, mixingForm,
        XtNfromVert, XLocked,
        XtNfromHoriz, YScroll,
        NULL);

    ZLocked = XtVaCreateManagedWidget("ZLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, YLocked,
        NULL);
    ZScroll = XtVaCreateManagedWidget("ZScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, YLocked,
        XtNfromHoriz, ZLocked,
        NULL);
    ZLabel = XtVaCreateManagedWidget("ZLabel", labelWidgetClass, mixingForm,
        XtNfromVert, YLocked,
        XtNfromHoriz, ZScroll,
        NULL);

    redLocked = XtVaCreateManagedWidget( "redLocked", labelWidgetClass,
        mixingForm, NULL );
    greenLocked = XtVaCreateManagedWidget( "greenLocked", labelWidgetClass,
        mixingForm,
        NULL );
    blueLocked = XtVaCreateManagedWidget( "blueLocked", labelWidgetClass,
        mixingForm, NULL );

    cursor = XCreateFontCursor( display, XC_sb_v_double_arrow );

    redScroll = XtVaCreateManagedWidget( "redScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    greenScroll = XtVaCreateManagedWidget( "greenScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    blueScroll = XtVaCreateManagedWidget( "blueScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    lockedScroll = XtVaCreateManagedWidget("lockedScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );

    redLabel = XtVaCreateManagedWidget( "redLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, mixedColor,
        XtNfromVert, mixedColor,
        NULL );
    greenLabel = XtVaCreateManagedWidget( "greenLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, redLocked,
        XtNfromVert, redLabel,
        NULL );
    blueLabel = XtVaCreateManagedWidget( "blueLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, greenLocked,
        XtNfromVert, greenLabel,
        NULL );

    quit = XtVaCreateManagedWidget("quit", commandWidgetClass, mixingForm,
        XtNfromVert, ZScroll,
        NULL);

    values.foreground = colors[PURE_RED].pixel;
    values.line_width = 2;
    values.line_style = LineOnOffDash;
    boxedChangedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth,
        &values);
    boxedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth | GCLineStyle,
        &values);

    XtVaGetValues(mixingForm, XtNbackground, &(values.foreground), NULL);
    unboxedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth, &values);

    original_background = values.foreground;

    bars_locked = 0;

    XtAddCallback(XScroll, XtNscrollProc, Scrolled, (XtPointer)XVAL);
    XtAddCallback(YScroll, XtNscrollProc, Scrolled, (XtPointer)YVAL);
    XtAddCallback(ZScroll, XtNscrollProc, Scrolled, (XtPointer)ZVAL);

    XtAddCallback(XScroll, XtNjumpProc, Update_CIE, (XtPointer)XVAL);
    XtAddCallback(YScroll, XtNjumpProc, Update_CIE, (XtPointer)YVAL);
    XtAddCallback(ZScroll, XtNjumpProc, Update_CIE, (XtPointer)ZVAL);

    XtAddCallback( redScroll, XtNscrollProc, Scrolled, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNscrollProc, Scrolled, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNscrollProc, Scrolled, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNscrollProc, Scrolled, ( XtPointer )LOCKED );

    XtAddCallback( redScroll, XtNjumpProc, Thumbed, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNjumpProc, Thumbed, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNjumpProc, Thumbed, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNjumpProc, Thumbed, ( XtPointer )LOCKED );

    XtAddCallback(tripleValue, XtNcallback, Set_Selection,
        (XtPointer)PRIMARY_COLOR);
    XtAddCallback(quit, XtNcallback, Quit, (XtPointer)toplevel);

    XtAppAddActions( app_context, cie_actions, XtNumber( cie_actions ));
    XtOverrideTranslations( cieColor,
        XtParseTranslationTable( cie_translations ));

    /* set the scrollbars to the initial mixed colour values */
    do_change = FALSE;
    CHANGE_X( );
    CHANGE_Y( );
    CHANGE_Z( );
    CHANGE_RED( MIX_COLOR );
    CHANGE_GREEN( MIX_COLOR );
    CHANGE_BLUE( MIX_COLOR );
    do_change = TRUE;
    XStoreColors(display, cmap, colors, 4);

    XtRealizeWidget( popup );
    XtPopup( popup, XtGrabNone );
}
